#html colorize pattern

import re
from ColorDelegator import ColorDelegator

defaulttagdefs=\
{
    'COMMENT':{'foreground':'#00b07f'},
    'KEYWORD':{'foreground':'#0000ff'},
    'ATTR': {'foreground':'#ff0000'},
    'PROPERTY': {'foreground':'#ff00ff'},
    'TODO':None
}

kwlist=['a', 'abbr', 'acronym', 'address', 'applet', 'area', 'b', 'base', 'basefont',
    'bdo', 'big', 'blockquote', 'body', 'br', 'button', 'caption', 'center', 'cite',
    'code', 'col', 'colgroup', 'dd', 'del', 'dfn', 'dir', 'div', 'dl', 'dt', 'em',
    'fieldset', 'font', 'form', 'frame', 'frameset', 'h1', 'h2', 'h3', 'h4', 'h5',
    'h6', 'head', 'hr', 'html', 'i', 'iframe', 'img', 'input', 'ins', 'isindex',
    'kbd', 'label', 'legend', 'li', 'link', 'map', 'menu', 'meta', 'noframes',
    'noscript', 'object', 'ol', 'optgroup', 'option', 'p', 'param', 'pre', 'q', 's',
    'samp', 'script', 'select', 'small', 'span', 'strike', 'strong', 'style', 'sub',
    'sup', 'table', 'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'title',
    'tr', 'tt', 'u', 'ul', 'var']
attrlist=['abbr', 'accept-charset', 'accept', 'accesskey', 'action', 'align', 'alink',
    'alt', 'archive', 'axis', 'background', 'bgcolor', 'border', 'cellspacing',
    'char', 'charoff', 'charset', 'checked', 'cite', 'class', 'classid', 'clear',
    'code', 'codebase', 'codetype', 'color', 'cols', 'colspan', 'compact', 'content',
    'coords', 'data', 'datatime', 'declare', 'defer', 'dir', 'disabled', 'enctype',
    'face', 'for', 'frame', 'frameborder', 'headers', 'height', 'href', 'hreflang',
    'hspace', 'http-equiv', 'id', 'ismap', 'label', 'lang', 'language', 'link',
    'longdesc', 'marginheight', 'marginwidth', 'maxlength', 'media', 'method',
    'multiple', 'name', 'nohref', 'noresize', 'noshade', 'nowrap', 'object',
    'onblur', 'onchange', 'onclick', 'ondbclick', 'onfocus', 'onkeydown', 'onkeypress',
    'onkeyup', 'onload', 'onmousedown', 'onmousemove', 'onmouseout', 'onmouseover',
    'onmouseup', 'onreset', 'onselect', 'onsubmit', 'onunload', 'profile', 'prompt',
    'readonly', 'rel', 'rev', 'rows', 'rowspan', 'rules', 'scheme', 'scope', 
    'scrolling', 'selected', 'shape', 'size', 'span', 'src', 'standby', 'start',
    'style', 'summary', 'tabindex', 'target', 'text', 'title', 'type', 'usemap',
    'valign', 'value', 'valuetype', 'version', 'vlink', 'vspace', 'width']

def any(name, list):
    return "(?P<%s>" % name + '|'.join(list) + ")"

def keys(name, list):
    return "(?P<%s>" % name + '|'.join(['</?\s*%s.*?>' % key for key in list]) + ")"

def attrs(list):
    return '|'.join(['%s' % key for key in list])

def make_pat():
    kw = keys("KEYWORD", kwlist)
    comment = any("COMMENT", [r"<!.*?>"])
    return comment + "|" + kw + "|" + any("SYNC", [r"\n"])

prog = re.compile(make_pat(), re.S|re.I)
keyprog = re.compile('(</?)\s*(\w+)(\s*.*?)(>)')
attrsprog = re.compile(r"\b"+attrs(attrlist)+r"\b", re.S|re.I)
attrprog=re.compile('(\w+)\s*((=)\s*([^"\'\s]+|"[^"]*"|\'[^\']*\'))?')

class HtmlColor(ColorDelegator):

    def __init__(self, prog=prog, tagdefs=defaulttagdefs):
        ColorDelegator.__init__(self, prog, tagdefs)

    def recolorize_main(self):
        next = "1.0"
        while 1:
            item = self.tag_nextrange("TODO", next)
            if not item:
                break
            head, tail = item
            self.tag_remove("SYNC", head, tail)
            item = self.tag_prevrange("SYNC", head)
            if item:
                head = item[1]
            else:
                head = "1.0"

            chars = ""
            next = head
            lines_to_get = 1
            ok = 0
            while not ok:
                mark = next
                next = self.index(mark + "+%d lines linestart" %
                                         lines_to_get)
                lines_to_get = min(lines_to_get * 2, 100)
                ok = "SYNC" in self.tag_names(next + "-1c")
                line = self.get(mark, next)
                ##print head, "get", mark, next, "->", `line`
                if not line:
                    return
                for tag in self.tagdefs.keys():
                    self.tag_remove(tag, mark, next)
                chars = chars + line
                m = self.prog.search(chars)
                while m:
                    for key, value in m.groupdict().items():
                        if value:
                            a, b = m.span(key)
                            if key=='COMMENT':
                                self.tag_add(key,
                                             head + "+%dc" % a,
                                             head + "+%dc" % b)
                            elif key=='KEYWORD':
                                t=keyprog.search(chars, a, b)
                                if t:
                                    for i in range(1, 5):
                                        if i==3: continue
                                        a, b=t.span(i)
                                        self.tag_add(key,
                                                     head + "+%dc" % a,
                                                     head + "+%dc" % b)
                                    begin, end=t.span(3)  #other text
                                    if begin>0 and end>0:
                                        t=attrsprog.search(chars, begin, end)
                                        while t:
                                            a, b=t.span()
                                            s=attrprog.search(chars, a, end)
                                            begin=s.span()[1]
                                            a, b=s.span(1)  #attr
                                            self.tag_add('ATTR',
                                                         head + "+%dc" % a,
                                                         head + "+%dc" % b)
                                            a, b=s.span(3)  #=
                                            if a>0 and b>0:
                                                self.tag_add('KEYWORD',
                                                             head + "+%dc" % a,
                                                             head + "+%dc" % b)
                                                a, b=s.span(4)  #property
                                                self.tag_add('PROPERTY',
                                                             head + "+%dc" % a,
                                                             head + "+%dc" % b)
                                            t=attrsprog.search(chars, begin, end)
                    m = self.prog.search(chars, m.end())
                if "SYNC" in self.tag_names(next + "-1c"):
                    head = next
                    chars = ""
                else:
                    ok = 0
                if not ok:
                    # We're in an inconsistent state, and the call to
                    # update may tell us to stop.  It may also change
                    # the correct value for "next" (since this is a
                    # line.col string, not a true mark).  So leave a
                    # crumb telling the next invocation to resume here
                    # in case update tells us to leave.
                    self.tag_add("TODO", next)
                self.update()
                if self.stop_colorizing:
                    return
