"""Provides access to configuration information"""

import os
import sys
from ConfigParser import ConfigParser, NoOptionError, NoSectionError

class IniFile(ConfigParser):

    def __init__(self, inifile=None):
        ConfigParser.__init__(self)
        self.inifile=inifile
        if inifile:
            self.read(inifile)

    def read(self, inifile):
        self.inifile=inifile
        if inifile:
            ConfigParser.read(self, inifile)

    def get(self, sec, options, default=None):
        """Get an option value for given section or return default"""
        try:
            return ConfigParser.get(self, sec, options, raw=0, vars=None)
        except (NoSectionError, NoOptionError):
            return default

    def getint(self, sec, option, default=0):
        try:
            return ConfigParser.getint(self, sec, option)
        except (NoSectionError, NoOptionError):
            return default
        
    def getfloat(self, sec, option, default=0.0):
        try:
            return ConfigParser.getfloat(self, sec, option)
        except (NoSectionError, NoOptionError):
            return default

    def getboolean(self, sec, option, default=0):
        try:
            return ConfigParser.getboolean(self, sec, option)
        except (NoSectionError, NoOptionError):
            return default

#    def set(self, section, option, value):
#        """Set an option."""
#        if not section or section == "DEFAULT":
#            sectdict = self._ConfigParser__defaults
#        else:
#            try:
#                sectdict = self._ConfigParser__sections[section]
#            except KeyError:
#                cursect = {'__name__': section}
#                self._ConfigParser__sections[section]=cursect
#                sectdict=cursect
#        option = self.optionxform(option)
#        sectdict[option] = value
#        self.save()

    def set(self, section, option, value):
        if not self.has_section(section):
            self.add_section(section)
        ConfigParser.set(self, section, option, value)
        self.save()

    def remove_section(self, section):
        res=ConfigParser.remove_section(self, section)
        if res:
            self.save()
        return res
    
    def remove_option(self, section, option):
        try:
            res=ConfigParser.remove_option(self, section, option)
            if res:
                self.save()
            return res
        except NoSectionError:
            return 0

    def add_section(self, section):
        ConfigParser.add_section(self, section)
        self.save()

    def save(self):
        fp=open(self.inifile, "w")
        self.write(fp)
        fp.close()
