from UndoDelegator import UndoDelegator
from WidgetRedirector import WidgetRedirector
from Tkinter import *

class LineUndoDelegator(UndoDelegator):

    linetext=None
    select=0

    def __init__(self, text=None, scroll=None, linenum=1):
        UndoDelegator.__init__(self)
        self.linenum = linenum
        self.text=text
        self.vbar=scroll
        if self.text:
            self.createlinetext()

    def insert(self, index, chars, tags=None):
        UndoDelegator.insert(self, index, chars, tags)
        self.changeline()

    def delete(self, index1, index2=None):
        UndoDelegator.delete(self, index1, index2)
        self.changeline()

    def changeline(self):
        if self.linenum:
            if not self.linetext:
                self.createlinetext()
            linetext=self.linetext
            linetext['state']=NORMAL
            tline=int(float(self.index('end-1c')))
            lline=int(float(linetext.index('end-1c')))
            if tline==lline: return
            if tline>lline:	#add lines num
                text=''
                for line in range(lline, tline):
                    text=text+'\n%d' % (line+1,)
                linetext.insert('end', text)
            else:
                linetext.delete('end-%dl' % (lline-tline,), 'end')
            self.linetext['state']=DISABLED

    def createlinetext(self):
        font=self.text['font']
        self.linetext=linetext=Text(self.text.master, width=5, relief=FLAT, fg='#999999',insertwidth=0, 
            font=font, cursor='right_ptr', exportselection=0)
        linetext.tag_config('sel', background='#ffffff', foreground='#999999')
        linetext.pack(side=LEFT, fill=Y)
        self.text.pack(side=RIGHT, fill=BOTH, expand=1)
        linetext.insert('end', '1')
        linetext['state']=DISABLED
        self.revbar=WidgetRedirector(self.vbar)
        self.vbar.config(command=self.myyview)
        self.oldset = self.revbar.register("set", self.set)
        linetext['yscrollcommand'] = self.vbar.set   #hoot scroll event
        linetext.bind('<ButtonPress>', self.on_mouse_press)
        linetext.bind('<Motion>',       self.on_mouse_motion)
        linetext.bind('<ButtonRelease>', self.on_mouse_release)

    def on_mouse_press(self, event):
        self.text.tag_remove(SEL, "1.0", END)
        self.select=1
        self.selstart=self.selend=int(float(self.text.index('@%d,%d' % (event.x, event.y))))
        self.selection_to(event)

    def on_mouse_motion(self, event):
        if self.select:
            self.selection_to(event)

    def on_mouse_release(self, event):
        if self.select:
            self.selection_to(event)
            self.select=0
    
    def myyview(self, moveto, offset):
        self.text.yview(moveto, offset)
        self.linetext.yview(moveto, offset)
        

    def set(self, lo, hi):
        self.text.yview('moveto', lo)
        self.linetext.yview('moveto', lo)
        self.oldset(lo, hi)

    def selection_to(self, event):
        newend=int(float(self.text.index('@%d,%d' % (event.x, event.y))))
        linestart=self.selstart
        lineend=self.selend
        if newend==linestart:
            newend=linestart+1
        if lineend>=linestart:   #down select
            if newend>=lineend:  #add select
                moreselect(self.text, lineend, newend)
            else:               #minus select
                reduceselect(self.text, newend, lineend)
        else:                   #up select
            if newend<lineend:  #more select
                moreselect(self.text, newend, lineend)
            else:               #minus select
                reduceselect(self.text, lineend, newend)
        self.selend=newend
#        self.text.tag_remove(SEL, "1.0", END)
#        self.text.tag_add(SEL, "%d.0" % (linestart,), "%d.0" % (lineend,))
        self.text.mark_set(INSERT, SEL_LAST)
        self.linetext.mark_set(SEL, "%d.0" % (lineend,))
        self.text.focus_set()

def moreselect(text, start, end):
    text.tag_add(SEL, "%d.0" % (start,), "%d.0" % (end,))

def reduceselect(text, start, end):
    text.tag_remove(SEL, "%d.0" % (start,), "%d.0" % (end,))
