import string
from Tkinter import *
import tkMessageBox

__all__=\
    [
        'menulist', 
        'keylist', 
        'eventlist', 
        'on_edit_autoindent_indent',
        'on_edit_autoindent_dedent',
        'on_edit_autoindent_tabify',
        'on_edit_autoindent_untabify',
        'on_edit_autoindent_toggle',
        'on_edit_autoindent_changetabwidth',
        'on_edit_autoindent_smart_backspace',
        'on_edit_autoindent_newlineindent',
        'on_edit_autoindent_smart_indent',
        'set_tabwidth',
        'get_tabwidth'
    ]

menulist=\
    [
        ('_Edit', 
            [
                ('Indent region\tCtrl+]', '<<indent-region>>'),
                ('Dedent region\tCtrl+[', '<<dedent-region>>'),
                ('Tabify region\tAlt+5', '<<tabify-region>>'),
                ('Untabify region\tAlt+6', '<<untabify-region>>'),
                ('Toggle tabs\tAlt+U', '<<toggle-tabs>>'),
                ('New indent width\tAlt+U', '<<change-tabwidth>>')
            ])
    ]
eventlist=\
    [
        ('<<indent-region>>',   'on_edit_autoindent_indent'),
        ('<<dedent-region>>',   'on_edit_autoindent_dedent'),
        ('<<tabify-region>>',   'on_edit_autoindent_tabify'),
        ('<<untabify-region>>', 'on_edit_autoindent_untabify'),
        ('<<toggle-tabs>>',     'on_edit_autoindent_toggle'),
        ('<<change-tabwidth>>',  'on_edit_autoindent_changetabwidth'),
        ('<<smart-backspace>>', 'on_edit_autoindent_smart_backspace'),
        ('<<newline-and-indent>>',  'on_edit_autoindent_newlineindent'),
        ('<<smart-indent>>',    'on_edit_autoindent_smart_indent')
    ]

keylist=\
    {
        '<<smart-backspace>>':      ['<Key-BackSpace>'],
        '<<newline-and-indent>>':   ['<Key-Return>'],
        '<<smart-indent>>':         ['<Key-Tab>'],
        '<<indent-region>>':        ['<Control-bracketright>'],
        '<<dedent-region>>':        ['<Control-bracketleft>'],
        '<<tabify-region>>':        ['<Alt-Key-5>'],
        '<<untabify-region>>':      ['<Alt-Key-6>'],
        '<<toggle-tabs>>':          ['<Alt-Key-t>'],
        '<<change-tabwidth>>':      ['<Alt-Key-u>']
    }

def init(self):
    self.usetabs = 1
    self.indentwidth = 4
    self.tabwidth = 4

    self.set_tabwidth(self.tabwidth)

def get_tabwidth(self):
    current = self.text['tabs'] or 8
    return int(current)

def set_tabwidth(self, newtabwidth):
    text = self.text
    if self.get_tabwidth() != newtabwidth:
        pixels = text.tk.call("font", "measure", text["font"],
                              "-displayof", text.master,
                              "n" * newtabwidth)
        text.configure(tabs=pixels)

def on_edit_autoindent_smart_backspace(self, event):
    text = self.text
    first, last = self.get_select()
    if first and last:
        text.delete(first, last)
        text.mark_set("insert", first)
        return "break"
    # Delete whitespace left, until hitting a real char or closest
    # preceding virtual tab stop.
    chars = text.get("insert linestart", "insert")
    if chars == '':
        if text.compare("insert", ">", "1.0"):
            # easy: delete preceding newline
            text.delete("insert-1c")
        else:
            text.bell()     # at start of buffer
        return "break"
    if  chars[-1] not in " \t":
        # easy: delete preceding real char
        text.delete("insert-1c")
        return "break"
    # Ick.  It may require *inserting* spaces if we back up over a
    # tab character!  This is written to be clear, not fast.
    expand, tabwidth = string.expandtabs, self.tabwidth
    have = len(expand(chars, tabwidth))
    assert have > 0
    want = int((have - 1) / self.indentwidth) * self.indentwidth
    ncharsdeleted = 0
    while 1:
        chars = chars[:-1]
        ncharsdeleted = ncharsdeleted + 1
        have = len(expand(chars, tabwidth))
        if have <= want or chars[-1] not in " \t":
            break
    text.undo_block_start()
    text.delete("insert-%dc" % ncharsdeleted, "insert")
    if have < want:
        text.insert("insert", ' ' * (want - have))
    text.undo_block_stop()
    return "break"

def on_edit_autoindent_smart_indent(self, event):
    # if intraline selection:
    #     delete it
    # elif multiline selection:
    #     do indent-region & return
    # indent one level
    text = self.text
    first, last = self.get_select()
    text.undo_block_start()
    try:
        if first and last:
            if index2line(first) != index2line(last):
                return self.indent_region_event(event)
            text.delete(first, last)
            text.mark_set("insert", first)
        prefix = text.get("insert linestart", "insert")
        raw, effective = classifyws(prefix, self.tabwidth)
        if raw == len(prefix):
            # only whitespace to the left
            reindent_to(self, effective + self.indentwidth)
        else:
            if self.usetabs:
                pad = '\t'
            else:
                effective = len(string.expandtabs(prefix,
                                                  self.tabwidth))
                n = self.indentwidth
                pad = ' ' * (n - effective % n)
            text.insert("insert", pad)
        text.see("insert")
        return "break"
    finally:
        text.undo_block_stop()
    return "break"

def on_edit_autoindent_newlineindent(self, event):
    text = self.text
    first, last = self.get_select()
    text.undo_block_start()
    try:
        if first and last:
            text.delete(first, last)
            text.mark_set("insert", first)
        line = text.get("insert linestart", "insert")
        i, n = 0, len(line)
        while i < n and line[i] in " \t":
            i = i+1
        if i == n:
            # the cursor is in or at leading indentation; just inject
            # an empty line at the start
            text.insert("insert linestart", '\n')
            return "break"
        indent = line[:i]
        # strip whitespace before insert point
        i = 0
        while line and line[-1] in " \t":
            line = line[:-1]
            i = i+1
        if i:
            text.delete("insert - %d chars" % i, "insert")
        # strip whitespace after insert point
        while text.get("insert") in " \t":
            text.delete("insert")
        # start new line
        text.insert("insert", '\n')
        text.insert("insert", indent)

        return "break"
    finally:
        text.see("insert")
        text.undo_block_stop()
    return "break"

def on_edit_autoindent_indent(self, event):
    head, tail, chars, lines = self.get_region()
    for pos in range(len(lines)):
        line = lines[pos]
        if line:
            raw, effective = classifyws(line, self.tabwidth)
            effective = effective + self.indentwidth
            lines[pos] = _make_blanks(self, effective) + line[raw:]
    self.set_region(head, tail, chars, lines)
    return "break"

def on_edit_autoindent_dedent(self, event):
    head, tail, chars, lines = self.get_region()
    for pos in range(len(lines)):
        line = lines[pos]
        if line:
            raw, effective = classifyws(line, self.tabwidth)
            effective = max(effective - self.indentwidth, 0)
            lines[pos] = _make_blanks(self, effective) + line[raw:]
    self.set_region(head, tail, chars, lines)
    return "break"

def on_edit_autoindent_tabify(self, event):
    head, tail, chars, lines = self.get_region()
    tabwidth = self.tabwidth
    for pos in range(len(lines)):
        line = lines[pos]
        if line:
            raw, effective = classifyws(line, tabwidth)
            ntabs, nspaces = divmod(effective, tabwidth)
            lines[pos] = '\t' * ntabs + ' ' * nspaces + line[raw:]
    self.set_region(head, tail, chars, lines)
    return "break"

def on_edit_autoindent_untabify(self, event):
    head, tail, chars, lines = self.get_region()
    tabwidth = self.tabwidth
    for pos in range(len(lines)):
        lines[pos] = string.expandtabs(lines[pos], tabwidth)
    self.set_region(head, tail, chars, lines)
    return "break"

def on_edit_autoindent_toggle(self, event):
    if self. tkMessageBox.askyesno(
          "Toggle tabs",
          "Turn tabs " + ("on", "off")[self.usetabs] + "?",
          parent=self.text):
        self.usetabs = not self.usetabs
    return "break"

# XXX this isn't bound to anything -- see class tabwidth comments
def on_edit_autoindent_changetabwidth(self, event):
    new = _asktabwidth(self)
    if new != self.tabwidth:
        self.tabwidth = new
    return "break"

# Make string that displays as n leading blanks.

def _make_blanks(self, n):
    if self.usetabs:
        ntabs, nspaces = divmod(n, self.tabwidth)
        return '\t' * ntabs + ' ' * nspaces
    else:
        return ' ' * n

# Delete from beginning of line to insert point, then reinsert
# column logical (meaning use tabs if appropriate) spaces.

def reindent_to(self, column):
    text = self.text
    text.undo_block_start()
    if text.compare("insert linestart", "!=", "insert"):
        text.delete("insert linestart", "insert")
    if column:
        text.insert("insert", _make_blanks(self, column))
    text.undo_block_stop()

def _asktabwidth(self):
    return self.tkMessageBox.askinteger(
        "Tab width",
        "Spaces per tab?",
        parent=self.text,
        initialvalue=self.tabwidth,
        minvalue=1,
        maxvalue=16) or self.tabwidth

# "line.col" -> line, as an int
def index2line(index):
    return int(float(index))

# Look at the leading whitespace in s.
# Return pair (# of leading ws characters,
#              effective # of leading blanks after expanding
#              tabs to width tabwidth)

def classifyws(s, tabwidth):
    raw = effective = 0
    for ch in s:
        if ch == ' ':
            raw = raw + 1
            effective = effective + 1
        elif ch == '\t':
            raw = raw + 1
            effective = (effective / tabwidth + 1) * tabwidth
        else:
            break
    return raw, effective

